@@ -43,13 +43,16 @@
 #include <stdlib.h>         // defines unlink() on some windows platforms(?)
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>         // for unlink()
+#endif
 #include <sys/types.h>      // for size_t
 #include <string>
-#endif
+#include <memory>           // for allocator
 #include <google/sparsetable>
 
 using STL_NAMESPACE::string;
+using STL_NAMESPACE::allocator;
 using GOOGLE_NAMESPACE::sparsetable;
+using GOOGLE_NAMESPACE::DEFAULT_SPARSEGROUP_SIZE;
 
 // Many sparsetable operations return a size_t.  Rather than have to
 // use PRIuS everywhere, we'll just cast to a "big enough" value.
@@ -443,7 +446,182 @@ void TestString() {
     out += snprintf(out, LEFT, "y[??] = %s\n", (*--it).c_str());
 }
 
-// The expected output from all of the above: TestInt() and TestString()
+// An instrumented allocator that keeps track of all calls to
+// allocate/deallocate/construct/destroy. It stores the number of times
+// they were called and the values they were called with. Such information is
+// stored in the following global variables.
+
+static size_t sum_allocate_bytes;
+static size_t sum_deallocate_bytes;
+
+void ResetAllocatorCounters() {
+  sum_allocate_bytes = 0;
+  sum_deallocate_bytes = 0;
+}
+
+template <class T> class instrumented_allocator {
+ public:
+  typedef T value_type;
+  typedef u_int16_t size_type;
+  typedef ptrdiff_t difference_type;
+
+  typedef T* pointer;
+  typedef const T* const_pointer;
+  typedef T& reference;
+  typedef const T& const_reference;
+
+  instrumented_allocator() {}
+  instrumented_allocator(const instrumented_allocator&) {}
+  ~instrumented_allocator() {}
+
+  pointer address(reference r) const  { return &r; }
+  const_pointer address(const_reference r) const  { return &r; }
+
+  pointer allocate(size_type n, const_pointer = 0) {
+    sum_allocate_bytes += n * sizeof(value_type);
+    return static_cast<pointer>(malloc(n * sizeof(value_type)));
+  }
+  void deallocate(pointer p, size_type n) {
+    sum_deallocate_bytes += n * sizeof(value_type);
+    free(p);
+  }
+
+  size_type max_size() const  {
+    return static_cast<size_type>(-1) / sizeof(value_type);
+  }
+
+  void construct(pointer p, const value_type& val) {
+    new(p) value_type(val);
+  }
+  void destroy(pointer p) {
+    p->~value_type();
+  }
+
+  template <class U>
+  explicit instrumented_allocator(const instrumented_allocator<U>&) {}
+
+  template<class U>
+  struct rebind {
+    typedef instrumented_allocator<U> other;
+  };
+
+ private:
+  void operator=(const instrumented_allocator&);
+};
+
+template<class T>
+inline bool operator==(const instrumented_allocator<T>&,
+                       const instrumented_allocator<T>&) {
+  return true;
+}
+
+template<class T>
+inline bool operator!=(const instrumented_allocator<T>&,
+                       const instrumented_allocator<T>&) {
+  return false;
+}
+
+// Test sparsetable with instrumented_allocator.
+void TestAllocator() {
+  out += snprintf(out, LEFT, "allocator test\n");
+
+  ResetAllocatorCounters();
+
+  // POD (int32) with instrumented_allocator.
+  typedef sparsetable<int, DEFAULT_SPARSEGROUP_SIZE,
+                      instrumented_allocator<int> > IntSparseTable;
+
+  IntSparseTable* s1 = new IntSparseTable(10000);
+  TEST(sum_allocate_bytes > 0);
+  for (int i = 0; i < 10000; ++i) {
+    s1->set(i, 0);
+  }
+  TEST(sum_allocate_bytes >= 10000 * sizeof(int));
+  ResetAllocatorCounters();
+  delete s1;
+  TEST(sum_deallocate_bytes >= 10000 * sizeof(int));
+
+  IntSparseTable* s2 = new IntSparseTable(1000);
+  IntSparseTable* s3 = new IntSparseTable(1000);
+
+  for (int i = 0; i < 1000; ++i) {
+    s2->set(i, 0);
+    s3->set(i, 0);
+  }
+  TEST(sum_allocate_bytes >= 2000 * sizeof(int));
+
+  ResetAllocatorCounters();
+  s3->clear();
+  TEST(sum_deallocate_bytes >= 1000 * sizeof(int));
+
+  ResetAllocatorCounters();
+  s2->swap(*s3);  // s2 is empty after the swap
+  s2->clear();
+  TEST(sum_deallocate_bytes < 1000 * sizeof(int));
+  for (int i = 0; i < s3->size(); ++i) {
+    s3->erase(i);
+  }
+  TEST(sum_deallocate_bytes >= 1000 * sizeof(int));
+  delete s2;
+  delete s3;
+
+  // POD (int) with default allocator.
+  sparsetable<int> x, y;
+  for (int s = 1000; s <= 40000; s += 1000) {
+    x.resize(s);
+    for (int i = 0; i < s; ++i) {
+      x.set(i, i + 1);
+    }
+    y = x;
+    for (int i = 0; i < s; ++i) {
+      y.erase(i);
+    }
+    y.swap(x);
+  }
+  TEST(x.num_nonempty() == 0);
+  out += snprintf(out, LEFT, "y[0]: %d\n", int(y[0]));
+  out += snprintf(out, LEFT, "y[39999]: %d\n", int(y[39999]));
+  y.clear();
+
+  // POD (int) with std allocator.
+  sparsetable<int, DEFAULT_SPARSEGROUP_SIZE, allocator<int> > u, v;
+  for (int s = 1000; s <= 40000; s += 1000) {
+    u.resize(s);
+    for (int i = 0; i < s; ++i) {
+      u.set(i, i + 1);
+    }
+    v = u;
+    for (int i = 0; i < s; ++i) {
+      v.erase(i);
+    }
+    v.swap(u);
+  }
+  TEST(u.num_nonempty() == 0);
+  out += snprintf(out, LEFT, "v[0]: %d\n", int(v[0]));
+  out += snprintf(out, LEFT, "v[39999]: %d\n", int(v[39999]));
+  v.clear();
+
+  // Non-POD (string) with default allocator.
+  sparsetable<string> a, b;
+  for (int s = 1000; s <= 40000; s += 1000) {
+    a.resize(s);
+    for (int i = 0; i < s; ++i) {
+      a.set(i, "aa");
+    }
+    b = a;
+    for (int i = 0; i < s; ++i) {
+      b.erase(i);
+    }
+    b.swap(a);
+  }
+  TEST(a.num_nonempty() == 0);
+  out += snprintf(out, LEFT, "b[0]: %s\n", b.get(0).c_str());
+  out += snprintf(out, LEFT, "b[39999]: %s\n", b.get(39999).c_str());
+  b.clear();
+}
+
+// The expected output from all of the above: TestInt(), TestString() and
+// TestAllocator().
 static const char g_expected[] = (
     "int test\n"
     "x[0]: 0\n"
@@ -675,17 +853,35 @@ static const char g_expected[] = (
     "y[??] = -13\n"
     "y[??] = -11\n"
     "y[??] = -10\n"
+    "allocator test\n"
+    "sum_allocate_bytes > 0? yes\n"
+    "sum_allocate_bytes >= 10000 * sizeof(int)? yes\n"
+    "sum_deallocate_bytes >= 10000 * sizeof(int)? yes\n"
+    "sum_allocate_bytes >= 2000 * sizeof(int)? yes\n"
+    "sum_deallocate_bytes >= 1000 * sizeof(int)? yes\n"
+    "sum_deallocate_bytes < 1000 * sizeof(int)? yes\n"
+    "sum_deallocate_bytes >= 1000 * sizeof(int)? yes\n"
+    "x.num_nonempty() == 0? yes\n"
+    "y[0]: 1\n"
+    "y[39999]: 40000\n"
+    "u.num_nonempty() == 0? yes\n"
+    "v[0]: 1\n"
+    "v[39999]: 40000\n"
+    "a.num_nonempty() == 0? yes\n"
+    "b[0]: aa\n"
+    "b[39999]: aa\n"
     );
 
 // defined at bottom of file for ease of maintainence
-int main(int argc, char **argv) {          // though we ignore the args
+int main(int /*argc*/, char ** /*argv*/) {
   TestInt();
   TestString();
+  TestAllocator();
 
   // Finally, check to see if our output (in out) is what it's supposed to be.
   const size_t r = sizeof(g_expected) - 1;
-  if ( r != out - outbuf ||               // output not the same size
-       memcmp(outbuf, g_expected, r) ) {  // or bytes differed
+  if ( r != static_cast<size_t>(out - outbuf) ||   // output not the same size
+       memcmp(outbuf, g_expected, r) ) {           // or bytes differed
     fprintf(stderr, "TESTS FAILED\n\nEXPECTED:\n\n%s\n\nACTUAL:\n\n%s\n\n",
             g_expected, outbuf);
     return 1;
