@@ -1,5 +1,5 @@
-#ifndef SPARSEHASH_WINDOWS_SPARSECONFIG_H__
-#define SPARSEHASH_WINDOWS_SPARSECONFIG_H__
+#ifndef SPARSEHASH_WINDOWS_SPARSECONFIG_H_
+#define SPARSEHASH_WINDOWS_SPARSECONFIG_H_
 
 /***
  *** These are #defines that autoheader puts in config.h.in that we
@@ -9,7 +9,10 @@
  *** config.h in installed header files, and we want to minimize the
  *** number of #defines we make so as to not pollute the namespace.
  ***/
-#define GOOGLE_NAMESPACE  google
+/*      NOTE: these are for internal use only.  Don't use these
+ *      #defines in your own programs!
+ */
+#define GOOGLE_NAMESPACE  ::google
 #define HASH_NAMESPACE  stdext
 #define HASH_FUN_H   <hash_map>
 #define SPARSEHASH_HASH  HASH_NAMESPACE::hash_compare
@@ -23,13 +26,7 @@
 #define HAVE_MEMCPY  1
 #define STL_NAMESPACE  std
 #define _END_GOOGLE_NAMESPACE_  }
-#define _START_GOOGLE_NAMESPACE_  namespace GOOGLE_NAMESPACE {
+#define _START_GOOGLE_NAMESPACE_  namespace google {
 
 
-// ---------------------------------------------------------------------
-// Extra stuff not found in config.h.include
-
-#define WIN32_LEAN_AND_MEAN  /* We always want minimal includes */
-#include <windows.h>         /* TODO(csilvers): do in every .h file instead? */
-
-#endif  /* SPARSEHASH_WINDOWS_SPARSECONFIG_H__ */
+#endif  /* SPARSEHASH_WINDOWS_SPARSECONFIG_H_ */
